import { Router, Config, I18n, Metadata } from '@zeedhi/core';
import { DialogService, Form, LoadingService, Modal, ModalService } from '@zeedhi/common';
import { AuthApi } from './AuthApi';
import { Util } from '../Support/Util';
import { MessageServices } from '../Support/MessageServices';
import { LibLocalStorage } from '../Support/LibLocalStorage';

export class AuthService {
    private constructor() {}

    private static instance: AuthService;

    public expiredSessionMethod: any = null;

    private expiredSessionModal!: Modal;

    private attemptsPassword: number = 0;

    public authenticationPage = 'autenticacao';

    public static getInstance(): AuthService {
        if (!AuthService.instance) {
            AuthService.instance = new AuthService();
        }
        return AuthService.instance;
    }

    public async startSession(userData: any) {
        try {
            this.setLocalStorageAfterLogin(userData);
            const { data } = await AuthApi.startSession(userData);
            this.setLoginOperatorprops(data);
            userData.loginUser = data.LOGIN_USER;
            this.setLocalStorageAfterLogin(userData);
            await this.buildMenu(data.VISIBLE_ELEMENTS, data.EXPIRED_ELEMENTS);
            this.redirectProduct();
        } catch (error) {
            MessageServices.defaultError(error.response);
        }
    }

    private setLoginOperatorprops(data: any) {
        LibLocalStorage.setItem('LOGIN_USER_NAME', data.USER_NAME);
        if (data.IS_SUPPORT_OPERATOR) {
            LibLocalStorage.setItem('IS_SUPPORT_OPERATOR', data.IS_SUPPORT_OPERATOR);
        }
    }

    public setLocalStorageAfterLogin(userData: any) {
        localStorage.setItem('LOGIN_TOKEN', userData.loginToken);
		localStorage.setItem('LOGIN_USER', userData.loginUser);
		localStorage.setItem('LOGIN_USER_NAME', userData.loginUserName);
		localStorage.setItem('LOGIN_HASH', userData.loginHash);
		localStorage.setItem('LOGIN_LANGUAGE', userData.loginLanguage);
		localStorage.setItem('LOGIN_KEEP_CONNECTED', userData.loginKeepConnected);
    }

    public setLocalStorageInLogout() {
        localStorage.removeItem('LOGIN_TOKEN');
		localStorage.removeItem('LOGIN_USER');
		localStorage.removeItem('LOGIN_USER_NAME');
		localStorage.removeItem('MENU');
    }

    public redirectProduct() {
        Router.replace('/');
    }

    public redirectLogin() {
        window.location.href = `${Config.env.product.url}${Config.env.loginRoutePath}`;
    }

    public async buildMenu(visibleElements: string[], expiredElements: string[]): Promise<any> {
        try {
            const response = await AuthApi.buildMenu(visibleElements, expiredElements);
            localStorage.setItem('MENU', JSON.stringify(response.data));
            return response;
        } catch (error) {
            return error;
        }
    }

    public showMessageExpiredMenu() {
        DialogService.show({
			name: 'success_dialog',
			type: 'warning',
			title: 'ALE_WARNING',
			text: I18n.translate('ALE_EXPIRED_MENU'),
		});
    }

    public async validateExpiredSession(this: AuthService) {
        try {
            const response = await AuthApi.validateExpiredSession(Config.env.product.sessionTime);
            if (response.data && response.data.IS_EXPIRED) {
                await this.createExpiredSessionModal();
                this.expiredSessionModal.show();
                LibLocalStorage.setItem('LOGIN_EXPIRED_SESSION', true);
            } else {
                if (this.expiredSessionMethod) {
                    clearTimeout(this.expiredSessionMethod);
                }
                const timeout = response.data.TIMEOUT || Config.env.product.sessionTime;
                Util.doAsync(this.validateExpiredSession.bind(this), timeout);
            }
        } catch (error) {
            MessageServices.defaultError(error.response);
        }
    }

    public factoryExpiredSessionMethod() {
        if (Config.env && Config.env.product && Config.env.product.sessionTime) {
            if (this.expiredSessionMethod) {
                clearTimeout(this.expiredSessionMethod);
            }
            this.expiredSessionMethod = Util.doAsync(this.validateExpiredSession.bind(this), Config.env.product.sessionTime);
        }
    }

    public async logout() {
        try {
            LoadingService.show();
            await AuthApi.logout();
            this.setLocalStorageInLogout();
            LibLocalStorage.setItem('LOGIN_EXPIRED_SESSION', false);
            this.redirectLogin();
            LoadingService.hide();
        } catch (error) {
            LoadingService.hide();
            MessageServices.defaultError(error.response);
        }
    }

    public async logoutConfirm() {
        const buttonsDoLogout = [
            {
                name: 'yes',
                component: 'ZdButton',
                label: I18n.translate('YES'),
                events: {
                    click: (() => {
                        DialogService.hide();
                        this.logout();
                    }),
                },
            },
            {
                name: 'no',
                component: 'ZdButton',
                label: I18n.translate('NO'),
                events: {
                    click: (() => {
                        DialogService.hide();
                    }),
                },
            },
        ];
        MessageServices.showMessage('ALE_LOGOUT', {}, 'warning', buttonsDoLogout);
    }

    private async createExpiredSessionModal() {
        this.expiredSessionModal = ModalService.create({
            name: 'expiredSessionModal',
            persistent: true,
            children: [
                {
                    name: 'expiredSessionModalHeader',
                    component: 'ZdHeader',
                    color: 'transparent',
                    cssClass: 'zd-mb-5',
                    padless: true,
                    elevation: 0,
                    leftSlot: [
                        {
                            name: 'expiredSessionModaltitleText',
                            component: 'ZdText',
                            text: 'Sessão Expirada',
                            cssClass: 'zd-theme-font-title',
                        },
                    ],
                    rightSlot: [],
                },
                {
                    name: 'expiredSessionform',
                    component: 'ZdForm',
                    children: [
                        {
                            name: 'PASSWORD',
                            label: 'Senha',
                            dense: false,
                            component: 'ZdPassword',
                            validations: {
                                required: {},
                            },
                            grid: {
                                sm: 12,
                            },
                            events: {
                                keydown: this.callValidateUser.bind(this),
                            },
                        },
                    ],
                },
                {
                    name: 'expiredSessionModalFooter',
                    component: 'ZdFooter',
                    color: 'transparent',
                    cssClass: 'zd-mt-5',
                    padless: true,
                    rightSlot: [
                        {
                            name: 'expiredSessionModalCancelButton',
                            component: 'ZdButton',
                            label: 'Sair',
                            outline: true,
                            events: {
                                click: () => {
                                    this.destroyExpiredSessionModal();
                                    this.logout();
                                },
                            },
                        },
                        {
                            name: 'expiredSessionModalSaveButton',
                            component: 'ZdButton',
                            label: 'Confirme',
                            events: {
                                click: () => {
                                    this.validateUser();
                                },
                            },
                        },
                    ],
                },
            ],
        });
    }

    private hideExpiredSessionModal() {
        this.expiredSessionModal.hide();
    }

    private destroyExpiredSessionModal() {
        this.expiredSessionModal.destroy();
    }

    private callValidateUser(param: any) {
        if (param.event.key === 'Enter') {
			this.validateUser();
		}
    }

    private async validateUser() {
        const expiredSessionform = Metadata.getInstance<Form>('expiredSessionform');
        LibLocalStorage.setItem('LOGIN_EXPIRED_SESSION', true);
        if (expiredSessionform.validate()) {
            try {
                LoadingService.show();
                const password = expiredSessionform.value.PASSWORD;
                const isSupportOperator = LibLocalStorage.getItem('IS_SUPPORT_OPERATOR');
                await AuthApi.validateUser(password, isSupportOperator);
                LibLocalStorage.setItem('LOGIN_EXPIRED_SESSION', false);
                this.attemptsPassword = 0;
                LoadingService.hide();
                this.destroyExpiredSessionModal();
            } catch (error) {
                LoadingService.hide();
                if (error.response.data && error.response.data.code && error.response.data.code === 5) {
                        this.attemptsPassword += 1;
                        if (this.attemptsPassword < 3) {
                            MessageServices.showMessage('ERR_ATTEMPT_PARTIAL', { attempts: this.attemptsPassword });
                        } else {
                            this.destroyExpiredSessionModal();
                            MessageServices.showMessage('ERR_ATTEMPT_END', {}, 'warning', this.buttonOkLogout());
                        }
                } else {
                    MessageServices.defaultError(error.response);
                }
            }
        }
    }

    public async checkExpiredSession() {
        if (LibLocalStorage.getItem('LOGIN_EXPIRED_SESSION')) {
            MessageServices.showMessage('ERR_EXPIRED_SESSION_LOGIN', {}, 'warning', this.buttonOkLogout());
        }
    }

    private buttonOkLogout() {
        return [
            {
                name: 'ok',
                component: 'ZdButton',
                label: I18n.translate('OK'),
                events: {
                    click: (() => {
                        DialogService.hide();
                        this.logout();
                    }),
                },
            },
        ];
    }

    public getCurrentPage() {
        if (process.env.NODE_ENV === 'production') {
            const routeParams = JSON.parse(<string>localStorage.getItem('routeParams'));
            if (routeParams) {
                return routeParams.page;
            }
        }
        const { searchParams } = new URL(window.location.href);
        return searchParams.get('page') || '';
	}
}
